@file:JvmName("TalkApiClientKt")

package com.kakao.sdk.talk

import android.net.Uri
import com.kakao.sdk.auth.network.TokenBasedApiInterceptor
import com.kakao.sdk.auth.network.kapiWithOAuth
import com.kakao.sdk.common.ApiFactory
import com.kakao.sdk.common.model.ApplicationInfo
import com.kakao.sdk.common.model.ContextInfo
import com.kakao.sdk.common.KakaoSdk
import com.kakao.sdk.common.KakaoJson
import com.kakao.sdk.template.DefaultTemplate
import com.kakao.sdk.network.RxApiInterceptor
import com.kakao.sdk.talk.model.*
import io.reactivex.Completable
import io.reactivex.Single

/**
 * @author kevin.kang. Created on 2018. 3. 30..
 */
class RxTalkApiClient(
    private val api: RxTalkApi = ApiFactory.kapiWithOAuth.create(RxTalkApi::class.java),
    val apiInterceptor: TokenBasedApiInterceptor = TokenBasedApiInterceptor.instance,
    private val applicationInfo: ApplicationInfo = KakaoSdk.applicationContextInfo,
    private val contextInfo: ContextInfo = KakaoSdk.applicationContextInfo
) {

    @JvmOverloads
    fun friends(
        offset: Int? = null,
        limit: Int? = null,
        order: Order? = null,
        secureResource: Boolean? = true
    ): Single<Friends<Friend>> = api.friends(secureResource, offset, limit, order)
        .compose(RxApiInterceptor.handleApiError())
        .compose(apiInterceptor.handleApiError())

    @JvmOverloads
    fun profile(
        secureResource: Boolean? = true
    ): Single<TalkProfile> =
        api.profile(secureResource)
            .compose(RxApiInterceptor.handleApiError())
            .compose(apiInterceptor.handleApiError())

    @JvmOverloads
    fun sendCustomMemo(templateId: Long, templateArgs: Map<String, String>? = null): Completable =
        api.sendCustomMemo(
            templateId,
            templateArgs
        ).compose(RxApiInterceptor.handleCompletableError())
            .compose(apiInterceptor.handleCompletableError())

    fun sendDefaultMemo(templateParams: DefaultTemplate): Completable =
        api.sendDefaultMemo(templateParams)
            .compose(RxApiInterceptor.handleCompletableError())
            .compose(apiInterceptor.handleCompletableError())

    @JvmOverloads
    fun sendScrapMemo(
        requestUrl: String,
        templateId: Long? = null,
        templateArgs: Map<String, String>? = null
    ): Completable =
        api.sendScrapMemo(requestUrl, templateId, templateArgs)
            .compose(RxApiInterceptor.handleCompletableError())
            .compose(apiInterceptor.handleCompletableError())

    @JvmOverloads
    fun sendCustomMessage(
        receiverUuids: List<String>,
        templateId: Long,
        templateArgs: Map<String, String>? = null
    ): Single<MessageSendResult> = api.sendCustomMessage(
        KakaoJson.toJson(receiverUuids),
        templateId,
        templateArgs
    ).compose(RxApiInterceptor.handleApiError())
        .compose(apiInterceptor.handleApiError())

    fun sendDefaultMessage(
        receiverUuids: List<String>,
        templateParams: DefaultTemplate
    ): Single<MessageSendResult> =
        api.sendDefaultMessage(KakaoJson.toJson(receiverUuids), templateParams)
            .compose(RxApiInterceptor.handleApiError())
            .compose(apiInterceptor.handleApiError())

    @JvmOverloads
    fun sendScrapMessage(
        receiverUuids: List<String>,
        requestUrl: String,
        templateId: Long? = null,
        templateArgs: Map<String, String>? = null
    ): Single<MessageSendResult> =
        api.sendScrapMessage(KakaoJson.toJson(receiverUuids), requestUrl, templateId, templateArgs)
            .compose(RxApiInterceptor.handleApiError())
            .compose(apiInterceptor.handleApiError())

    @JvmOverloads
    fun plusFriends(publicIds: List<String>? = null): Single<PlusFriendsRelations> =
        api.plusFriends(
            if (publicIds == null) null else KakaoJson.toJson(
                publicIds
            )
        ).compose(RxApiInterceptor.handleApiError())
            .compose(apiInterceptor.handleApiError())

    fun addFriendUrl(plusFriendId: String): Uri {
        return baseUri(appKey = applicationInfo.clientId, kaHeader = contextInfo.kaHeader)
            .path("$plusFriendId/${Constants.FRIEND}").build()
    }

    fun chatUrl(plusFriendId: String): Uri {
        return baseUri(appKey = applicationInfo.clientId, kaHeader = contextInfo.kaHeader)
            .path("$plusFriendId/${Constants.CHAT}").build()
    }

    private fun baseUri(appKey: String, kaHeader: String): Uri.Builder {
        return Uri.Builder().scheme(com.kakao.sdk.common.Constants.SCHEME)
            .authority(KakaoSdk.serverHosts.plusFriend)
            .appendQueryParameter(com.kakao.sdk.common.Constants.APP_KEY, appKey)
            .appendQueryParameter(Constants.KAKAO_AGENT, kaHeader)
            .appendQueryParameter(Constants.API_VER, Constants.API_VER_10)
    }

    companion object {
        @JvmStatic
        val instance by lazy { TalkApiClient.rx }
    }
}

val TalkApiClient.Companion.rx by lazy { RxTalkApiClient() }
